USE omop4;
GO

/* *** BLOCK 1: Extract relevant data element FROM source data and store in #tmp *** */
IF OBJECT_ID('tempdb..#tmp') IS NOT NULL
	DROP TABLE #tmp;
CREATE TABLE #tmp (
	person_id INT
	,eventDate DATE
	,eventConceptId VARCHAR(100)
	,eventType VARCHAR(100)
	,eventNumValue FLOAT NULL
	,eventStringValue VARCHAR(100) NULL
	);

INSERT INTO #tmp
SELECT DISTINCT *
FROM (
	SELECT P0.person_id
		,U.eventDate
		,U.eventConceptId
		,U.eventType
		,U.eventNumValue
		,U.eventStringValue
	FROM omop4.eMergeCohort P0
	LEFT JOIN (

		SELECT person_id AS person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxCtrlExcl' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21001477','21001683','21013483','21001688','21001692','21013769','21001656','21001685','21001475','21001476','21001695','21001696','21001697','21001698','21001693','21001694','21005668','439838','440417','443537','193512','196715','318775','435565','444247','4207615','40480461','439314','4028057','4077218','4121649','4179911','4203836','4323778','40480555','40479606','199837','254662','4084941','4111872','4234997','37219872','37622456','37622459','4177681','35909549','37019359','37622479','36110644','35909568')
		
		UNION ALL
    	SELECT person_id AS person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxProcCoronaryHeartDis' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21001456','21001462','21001467','21004432','21001447','21001448','21001454','21001455','21001584','21013849','21001453','21004431','21011303','312327','316995','319038','434376','436706','438170','438438','438447','439693','441579','444406','4108669','4185932','319844','314666','315296','35205180','35205190','35205384','35205168','35205189','35205182')
		
		UNION ALL
		SELECT person_id AS person_id
			,procedure_date AS eventDate
			,procedure_concept_id AS eventConceptId
			,'DxProcCoronaryHeartDis' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.procedure_occurrence
		WHERE procedure_concept_id in ('2108631')
		
		/* NLP note part */
	  	UNION ALL
		SELECT person_id AS person_id
			,noteDate AS eventDate
			,NULL AS eventConceptId
			,'NoteVte' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.nlpNote
		WHERE AlgVar = 'vte.noteMention'
		
		) U ON P0.person_id = U.person_id
	) Utotal;

/* *** BLOCK 2: Get algorithm related variables *** */
IF OBJECT_ID('tempdb..#AlgVar') IS NOT NULL
	DROP TABLE #AlgVar;

CREATE TABLE #AlgVar (
	person_id INT NOT NULL
	,diseaseName VARCHAR(50) NOT NULL
	,caseControl VARCHAR(50) NOT NULL
	,C00 INT NOT NULL
	,C01 INT NOT NULL
	);

INSERT INTO #AlgVar
SELECT DISTINCT J.person_id
	,'Temp' AS diseaseName
	,'CASE/CONTROL' AS caseControl
	,ISNULL(J0.noteVteCnt,0) AS C00
	,ISNULL(J1.dxCtrlExclCnt, 0) AS C01
FROM omop4.eMergeCohort J

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS noteVteCnt
	FROM #tmp
	WHERE eventType = 'NoteVte'
	GROUP BY person_id
	) J0 
ON J.person_id = J0.person_id
LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxCtrlExclCnt
	FROM #tmp
	WHERE eventType = 'DxCtrlExcl'
	GROUP BY person_id
	) J1
ON J.person_id = J1.person_id;
/* *** BLOCK 3: Phenotyping **** */

IF OBJECT_ID('tempdb..#phenotype') IS NOT NULL
	DROP TABLE #phenotype;
CREATE TABLE #phenotype (
	person_id INT
	,pCase VARCHAR(50)
	,pCtrl VARCHAR(50)
	);

INSERT INTO #phenotype
SELECT DISTINCT CL.person_id /* CL has duplicate person_id */
	,ISNULL(U1.phenotype, 'NO') AS pCase
	,ISNULL(U2.phenotype, 'NO') AS pCtrl
FROM omop4.eMergeCohort CL
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C00 > 0
			)
	) U1 ON CL.person_id = U1.person_id
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C00=0
			AND C01=0
			)
	) U2 ON CL.person_id = U2.person_id;

/* *** BLOCK 4: Write each patient phenotyping result (PR) to a table *** */
IF COL_LENGTH('$tablePR', 'VteCase') IS NULL
	ALTER TABLE $tablePR ADD VteCase VARCHAR(50);
IF COL_LENGTH('$tablePR', 'VteCtrl') IS NULL
	ALTER TABLE $tablePR ADD VteCtrl VARCHAR(50);

UPDATE $tablePR
SET VteCase = (
		SELECT pCase
		FROM #phenotype
		WHERE #phenotype.person_id = $tablePR.person_id
		)
	,VteCtrl = (
		SELECT pCtrl
		FROM #phenotype
		WHERE #phenotype.person_id = $tablePR.person_id
		);

/* *** BLOCK 5: Get count for each phenotype *** */
SELECT U1.CaseCnt
	,U2.CtrlCnt
FROM (
	SELECT Count(DISTINCT person_id) AS CaseCnt
	FROM #phenotype
	WHERE pCase = 'YES'
	) U1
	,(
		SELECT Count(DISTINCT person_id) AS CtrlCnt
		FROM #phenotype
		WHERE pCtrl = 'YES'
		) U2;

/* *** For phekb submission *** */
SELECT person_id AS eMERGE_ID
	,CASE WHEN pCase = 'YES' THEN 'C49152'
		WHEN pCtrl = 'YES' THEN 'C28143'
		WHEN pCase = 'NO' AND pCtrl = 'NO' THEN 'NO' END AS Case_Control
FROM #phenotype
ORDER BY eMERGE_ID
;